@@ -20,8 +20,11 @@
 #include "loader.h"
 
 #include "loaders/loader-3ds.h"
+#include "loaders/loader-ac.h"
+#include "loaders/loader-md2.h"
 
 #include "model.h"
+#include "debug.h"
 
 
 Loader::Loader(Model* m, LOD* l, const QString& file)
@@ -37,6 +40,26 @@ Loader::~Loader()
 
 Loader* Loader::createLoader(Model* m, LOD* l, const QString& filename)
 {
-  return (Loader*)(new Loader3DS(m, l, filename));
+  if(filename.lower().endsWith(".3ds"))
+  {
+    return (Loader*)(new Loader3DS(m, l, filename));
+  }
+  else if(filename.lower().endsWith(".ac"))
+  {
+    return (Loader*)(new LoaderAC(m, l, filename));
+  }
+  else if(filename.lower().endsWith(".md2"))
+  {
+    return (Loader*)(new LoaderMD2(m, l, filename));
+  }
+  else
+  {
+    boError() << k_funcinfo << "unrecognized file format of " << filename << endl;
+    return 0;
+  }
 }
 
+/*
+ * vim: et sw=2
+ */
+
