@@ -21,6 +21,8 @@
 
 #include <qobject.h>
 
+#include "global.h"
+
 class Cell;
 class PlayerIO;
 class QString;
@@ -43,6 +45,8 @@ class BoGroundRendererStatistics
 public:
 	BoGroundRendererStatistics()
 	{
+		mMinDistance = 0.0;
+		mMaxDistance = 0.0;
 		clear();
 	}
 	~BoGroundRendererStatistics()
@@ -54,6 +58,7 @@ public:
 		mRenderedCells = 0;
 		mRenderedQuads = 0;
 		mUsedTextures = 0;
+		// Do not clear min/max distances here!
 	}
 
 	QString statisticsData() const;
@@ -70,6 +75,14 @@ public:
 	{
 		mUsedTextures = t;
 	}
+	void setMinDistance(float d)
+	{
+		mMinDistance = d;
+	}
+	void setMaxDistance(float d)
+	{
+		mMaxDistance = d;
+	}
 
 	unsigned int renderedCells() const
 	{
@@ -83,10 +96,20 @@ public:
 	{
 		return mUsedTextures;
 	}
+	float minDistance() const
+	{
+		return mMinDistance;
+	}
+	float maxDistance() const
+	{
+		return mMaxDistance;
+	}
 private:
 	unsigned int mRenderedCells;
 	unsigned int mRenderedQuads;
 	unsigned int mUsedTextures;
+	float mMinDistance;
+	float mMaxDistance;
 };
 
 /**
@@ -142,7 +165,7 @@ public:
 	 * could have been rendered three times, always with a different
 	 * texture, but will occur only once here!)
 	 **/
-	unsigned int renderCells(const BosonMap* map);
+	unsigned int renderCells(const BosonMap* map, RenderFlags flags);
 
 	void setLocalPlayerIO(PlayerIO* p)
 	{
@@ -169,6 +192,7 @@ public:
 
 	virtual QString debugStringForPoint(const BoVector3Fixed& pos) const
 	{
+		Q_UNUSED(pos);
 		return QString::null;
 	}
 
@@ -185,6 +209,13 @@ public:
 	 **/
 	virtual void cellFogChanged(int x1, int y1, int x2, int y2);
 
+	/**
+	 * Called when explored status of a cell or cells changes.
+	 * x1, y1, x2, y2 define rectangle which includes all the changed cells.
+	 * E.g. if x1 = x2 = 3 and y1 = y2 = 4, then only cell at (3; 4) changed.
+	 **/
+	virtual void cellExploredChanged(int x1, int y1, int x2, int y2);
+
 	/**
 	 * Called when height of a corner or corners changes.
 	 * x1, y1, x2, y2 define rectangle which includes all the changed corners.
@@ -208,18 +239,6 @@ public:
 	virtual bool usable() const;
 
 protected:
-	/**
-	 * This generates an array of visible cells for the @p playerIO. It works
-	 * on the list previously created by @ref generateCellList. (i.e. on
-	 * @ref renderCells)
-	 *
-	 * This mainly checks for whether the cells are fogged.
-	 *
-	 * Note that you _MUST_ delete[] the array when you are done using it!
-	 * @param cellCount The number of cells in the array is returned here.
-	 **/
-	int* createVisibleCellList(int* cellCount, PlayerIO* playerIO);
-
 	void setRenderCells(int* renderCells, int renderCellsSize);
 	void setRenderCellsCount(unsigned int count);
 
@@ -273,7 +292,7 @@ protected:
 	 * Actually render the currently visible cells. Implemented by the
 	 * plugin.
 	 **/
-	virtual void renderVisibleCells(int* cells, unsigned int cellsCount, const BosonMap* map) = 0;
+	virtual void renderVisibleCells(int* cells, unsigned int cellsCount, const BosonMap* map, RenderFlags flags) = 0;
 	virtual void renderVisibleCellsStart(const BosonMap* map) { Q_UNUSED(map) }
 	virtual void renderVisibleCellsStop(const BosonMap* map) { Q_UNUSED(map) }
 
