@@ -673,7 +673,6 @@ void BoUfoAction::init(BoUfoActionCollection* parent, const QString& text, const
 void BoUfoAction::initShortcut()
 {
  if (qstrcmp(name(), "unnamed") == 0) {
-	boDebug() << k_funcinfo << "won't initialize shortcut for unnamed action" << endl;
 	return;
  }
  if (!mParentCollection) {
@@ -1304,7 +1303,6 @@ void BoUfoActionCollection::remove(BoUfoAction* action, bool deleteIt)
 
 void BoUfoActionCollection::clearActions()
 {
- boDebug() << k_funcinfo << endl;
  if (parent() && parent()->inherits("BoUfoManager")) {
 	BoUfoManager* m = (BoUfoManager*)parent();
 	m->setMenuBarData(0);
@@ -1531,7 +1529,6 @@ void BoUfoMenuBar::createMenuBar()
 	d->mMenuBar->getParent()->setIndexOf(d->mMenuBar, 0);
  }
 
- boDebug() << k_funcinfo << "creating submenus" << endl;
  createUfoMenuBarSubMenu(d->mMenuBar);
 }
 
@@ -1604,7 +1601,6 @@ void BoUfoToolBar::createToolBar()
  d->mUfoManager->toolBarContentWidget()->ufoWidget()->setFont(d->mUfoManager->rootPane()->getFont());
  d->mUfoManager->toolBarContentWidget()->setVisible(d->mIsVisible);
 
- boDebug() << k_funcinfo << "creating toolbar submenus" << endl;
  createUfoToolBarSubMenu(d->mUfoManager->toolBarContentWidget()->ufoWidget());
 }
 
