@@ -27,6 +27,7 @@
 #include "bosonprofiling.h"
 #include "boitemlist.h"
 #include "cell.h"
+#include "unitorder.h"
 
 #include <qdom.h>
 
@@ -58,7 +59,7 @@ UnitMover::~UnitMover()
 
 bool UnitMover::init()
 {
- unit()->setWork(UnitBase::WorkIdle);
+ unit()->setAdvanceWork(UnitBase::WorkIdle);
  return true;
 }
 
@@ -178,57 +179,12 @@ void UnitMover::initCellIntersectionTable()
  }
 }
 
-void UnitMover::advanceMove(unsigned int advanceCallsCount)
+bool UnitMover::advancedRectWillBeOnCanvas() const
 {
- advanceMoveInternal(advanceCallsCount);
- advanceMoveCheck();
-}
-
-void UnitMover::advanceFollow(unsigned int advanceCallsCount)
-{
- if (advanceCallsCount % 5 != 0) {
-	return;
- }
- BosonProfiler profiler("advanceFollow");
- if (!unit()->target()) {
-	boWarning() << k_funcinfo << "cannot follow NULL unit" << endl;
-	unit()->stopAttacking();  // stopAttacking should maybe be renamed to stopEverything
-			//  or just stop because it's used in several places to stop unit from
-			//  doing whatever it does.
-	return;
- }
- if (unit()->target()->isDestroyed()) {
-	boDebug(401) << k_funcinfo << "Unit is destroyed!" << endl;
-	unit()->stopAttacking();
-	return;
- }
-// if (!isNextTo(target())) {  // This doesn't work for some reason :-(  Dunno why.
- if (QMAX(QABS(unit()->x() - unit()->target()->x()), QABS(unit()->y() - unit()->target()->y())) > 1.0f) {
-	// We're not next to unit
-	// AB: warning - this does a lookup on all items and therefore is slow!
-	// --> but we need it as a simple test on the pointer causes trouble if
-	// that pointer is already deleted. any nice solutions?
-	if (!canvas()->allItems()->contains(unit()->target())) {
-		boDebug(401) << k_funcinfo << "Unit seems to be destroyed!" << endl;
-		unit()->stopAttacking();
-		return;
-	}
-	boDebug(401) << k_funcinfo << "unit (" << unit()->target()->id() << ") not in range - moving..." << endl;
-	if (!unit()->moveTo(unit()->target()->x(), unit()->target()->y(), 1)) {
-		unit()->setWork(UnitBase::WorkIdle);
-	} else {
-		unit()->setAdvanceWork(UnitBase::WorkMove);
-	}
- }
- // Do nothing (unit is in range)
-}
-
-
-void UnitMover::advanceMoveCheck()
-{
- PROFILE_METHOD;
+#define VERBOSE_ERROR 1
  // Check if top-left point of the unit will be on canvas after moving
  if (!canvas()->onCanvas(unit()->boundingRectAdvanced().topLeft())) {
+#if VERBOSE_ERROR
 	BoVector2Fixed point = unit()->boundingRectAdvanced().topLeft();
 	QString problem;
 	if (!canvas()->onCanvas(BoVector2Fixed(0, point.y()))) {
@@ -251,11 +207,14 @@ void UnitMover::advanceMoveCheck()
 	boError(401) << k_funcinfo << "leaving unit a current topleft pos: ("
 			<< unit()->boundingRect().topLeft().x() << ";"
 			<< unit()->boundingRect().topLeft().y() << ")" << endl;
-	unit()->stopMoving();
-	return;
+#endif //VERBOSE_ERROR
+
+	return false;
  }
- // Check if bottom-right point of the unit will be on canvas after moving
+
+// Check if bottom-right point of the unit will be on canvas after moving
  if (!canvas()->onCanvas(unit()->boundingRectAdvanced().bottomRight())) {
+#if VERBOSE_ERROR
 	BoVector2Fixed point = unit()->boundingRectAdvanced().bottomRight();
 	QString problem;
 	if (!canvas()->onCanvas(BoVector2Fixed(0, point.y()))) {
@@ -284,51 +243,120 @@ void UnitMover::advanceMoveCheck()
 	boError(401) << k_funcinfo << "leaving unit a current bottomright pos: ("
 			<< unit()->boundingRect().bottomRight().x() << ";"
 			<< unit()->boundingRect().bottomRight().y() << ")" << endl;
-	unit()->stopMoving();
+#endif // VERBOSE_ERROR
+
+	return false;
+ }
+
+#undef VERBOSE_ERROR
+ return true;
+}
+
+void UnitMover::advanceMove(unsigned int advanceCallsCount)
+{
+ // actually move
+ advanceMoveInternal(advanceCallsCount);
+
+ // check whether velocity/whatever of this unit causes it to go off the map or
+ // collide with other units. if so, revert the changes and stop moving.
+ //
+ // advanceMoveInternal() should also ensure these things, so advanceMoveCheck()
+ // should be a noop in theory - but since moving is a very complex task, it
+ // usually is required anyway.
+ advanceMoveCheck();
+}
+
+void UnitMover::advanceFollow(unsigned int advanceCallsCount)
+{
+ if (advanceCallsCount % 5 != 0) {
 	return;
  }
+ BosonProfiler profiler("advanceFollow");
+
+ UnitFollowOrder* followorder = (UnitFollowOrder*)unit()->currentOrder();
+ Unit* target = followorder->target();
+ if (!target) {
+	boWarning() << k_funcinfo << "cannot follow NULL unit" << endl;
+	unit()->currentSuborderDone(false);
+	return;
+ }
+ if (target->isDestroyed()) {
+	boDebug(401) << k_funcinfo << "Unit is destroyed!" << endl;
+	unit()->currentSuborderDone(true);
+	return;
+ }
+
+ if (QMAX(QABS(unit()->x() - target->x()), QABS(unit()->y() - target->y())) > followorder->distance()+1) {
+	// We're too far from the followed unit
+	// AB: warning - this does a lookup on all items and therefore is slow!
+	// --> but we need it as a simple test on the pointer causes trouble if
+	// that pointer is already deleted. any nice solutions?
+	if (!canvas()->allItems()->contains(target)) {
+		boDebug(401) << k_funcinfo << "Unit seems to be destroyed!" << endl;
+		unit()->currentSuborderDone(true);
+		return;
+	}
+	boDebug(401) << k_funcinfo << "unit (" << target->id() << ") not in range - moving..." << endl;
+	if (!unit()->addCurrentSuborder(new UnitMoveToUnitOrder(target, followorder->distance(), false))) {
+		unit()->currentSuborderDone(false);
+	}
+	return;
+ }
+ // Do nothing (unit is in range)
+}
+
+void UnitMover::advanceMoveCheck()
+{
+ PROFILE_METHOD;
+ if (!advancedRectWillBeOnCanvas()) {
+	unit()->clearOrders();
+ }
 }
 
-void UnitMover::turnTo()
+bool UnitMover::turnTo()
 {
  bofixed xspeed = unit()->xVelocity();
  bofixed yspeed = unit()->yVelocity();
  // Set correct rotation
  // Try to find rotation fast first
  if ((xspeed == 0) && (yspeed < 0)) { // North
-	unit()->turnTo(0);
+	return unit()->turnTo(0);
  } else if ((xspeed > 0) && (yspeed == 0)) { // East
-	unit()->turnTo(90);
+	return unit()->turnTo(90);
  } else if ((xspeed == 0) && (yspeed > 0)) { // South
-	unit()->turnTo(180);
+	return unit()->turnTo(180);
  } else if ((xspeed < 0) && (yspeed == 0)) { // West
-	unit()->turnTo(270);
+	return unit()->turnTo(270);
  } else if (QABS(xspeed) == QABS(yspeed)) {
 	if ((xspeed > 0) && (yspeed < 0)) { // NE
-		unit()->turnTo(45);
+		return unit()->turnTo(45);
 	} else if ((xspeed > 0) && (yspeed > 0)) { // SE
-		unit()->turnTo(135);
+		return unit()->turnTo(135);
 	} else if ((xspeed < 0) && (yspeed > 0)) { // SW
-		unit()->turnTo(225);
+		return unit()->turnTo(225);
 	} else if ((xspeed < 0) && (yspeed < 0)) { // NW
-		unit()->turnTo(315);
+		return unit()->turnTo(315);
 	} else if(xspeed == 0 && yspeed == 0) {
 //		boDebug() << k_funcinfo << "xspeed == 0 and yspeed == 0" << endl;
 	}
  } else {
 	// Slow way - calculate direction
-	unit()->turnTo((int)Bo3dTools::rotationToPoint(xspeed, yspeed));
+	return unit()->turnTo(Bo3dTools::rotationToPoint(xspeed, yspeed));
  }
+
+ return false;
 }
 
 bool UnitMover::attackEnemyUnitsInRangeWhileMoving()
 {
  PROFILE_METHOD
- // Don't attack other units unless work is WorkMove
- if (work() != UnitBase::WorkMove) {
+ if (!pathInfo()->moveAttacking) {
 	return false;
  }
- if (pathInfo()->moveAttacking && unit()->attackEnemyUnitsInRange()) {
+
+ UnitMoveOrderData* orderdata = (UnitMoveOrderData*)unit()->currentOrderData();
+ orderdata->target = unit()->attackEnemyUnitsInRange(orderdata->target);
+ if (orderdata->target) {
 	boDebug(401) << k_funcinfo << "unit " << id() << ": Enemy units found in range, attacking" << endl;
 	if (unit()->isFlying()) {
 		// Don't stop moving
@@ -342,6 +370,29 @@ bool UnitMover::attackEnemyUnitsInRangeWhileMoving()
  return false;
 }
 
+void UnitMover::stopMoving(bool success)
+{
+// boDebug() << k_funcinfo << endl;
+ unit()->clearPathPoints();
+
+ // Release highlevel path here once we cache them
+
+ if (!unit()->isFlying()) {
+	unit()->setMovingStatus(UnitBase::Standing);
+	unit()->setVelocity(0.0, 0.0, 0.0);
+
+	if (pathInfo() && pathInfo()->slowDownAtDest) {
+		unit()->setSpeed(0);
+	}
+ }
+ UnitOrder* order = unit()->currentOrder();
+ if (order && (order->type() == UnitOrder::Move || order->type() == UnitOrder::MoveToUnit)) {
+	unit()->currentSuborderDone(success);
+ }
+}
+
+
+
 void UnitMover::addUpgrade(const UpgradeProperties* upgrade)
 {
  changeUpgrades(upgrade);
@@ -407,9 +458,21 @@ bool UnitMoverLand::init()
 void UnitMoverLand::advanceMoveInternal(unsigned int advanceCallsCount)
 {
  BosonProfiler profiler("advanceMoveInternal");
- //boDebug(401) << k_funcinfo << endl;
+ advanceMoveInternal2(advanceCallsCount);
+ // advanceMoveInternal3() is called only conditionally by advanceMoveInternal2()
 
- if (pathInfo()->waiting) {
+ advanceMoveInternal4(advanceCallsCount);
+}
+
+// TODO: rename. maybe "calculate path", "update path", "ensure path", "ensure
+//       pathpoints" or something like that.
+//       -> it does not actually move, but will calculate path if necessary, and
+//          will abort moving if necessary.
+//
+void UnitMoverLand::advanceMoveInternal2(unsigned int advanceCallsCount)
+{
+ BO_CHECK_NULL_RET(pathInfo());
+ if (pathInfo()->waiting != 0) {
 	// If path is blocked and we're waiting, then there's no point in
 	//  recalculating velocity and other stuff every advance call
 	// TODO: check for enemies
@@ -421,98 +484,64 @@ void UnitMoverLand::advanceMoveInternal(unsigned int advanceCallsCount)
  if (unit()->maxSpeed() == 0) {
 	// If unit's max speed is 0, it cannot move
 	boError(401) << k_funcinfo << "unit " << id() << ": maxspeed == 0" << endl;
-	unit()->stopMoving();
-	unit()->setMovingStatus(UnitBase::Standing);
+	stopMoving(false);
 	return;
  }
 
  // Check if path is already found or not
- if (unit()->movingStatus() == UnitBase::MustSearch) {
+ if (unit()->movingStatus() == UnitBase::MustSearchPath) {
 	// Path is not yet searched
 	// If we're moving with attacking, first check for any enemies in the range.
 	if (attackEnemyUnitsInRangeWhileMoving()) {
 		return;
 	}
 	// If there aren't any enemies, find new path
-	if (!newPath()) {
+	if (!calculateNewPath()) {
 		// No path was found
-		unit()->stopMoving();
+		stopMoving(false);
 		return;
 	}
  }
-
- // Make sure we have pathpoints
- if (unit()->pathPointCount() == 0) {
-	if (pathInfo()->result == BosonPath::OutOfRange) {
-		// New pathfinding query has to made, because last returned path was
-		//  only partial.
-		if (!newPath()) {
-			// Probably no path could be found or we're already at destination point
-			unit()->stopMoving();
+ if (unit()->currentOrder()->type() == UnitOrder::MoveToUnit) {
+	const BoVector2Fixed& targetpos = ((UnitMoveToUnitOrder*)unit()->currentOrder())->target()->center();
+	if (((UnitMoveToUnitOrderData*)unit()->currentOrderData())->lastTargetPos.dotProduct(targetpos) > 5*5) {
+		// Update path
+		if (!calculateNewPath()) {
+			// No path was found
+			stopMoving(false);
 			return;
 		}
-	} else {
-		// TODO: rotate a bit randomly
-		// TODO: support multiple waypoints
-		unit()->stopMoving();
-		return;
 	}
  }
 
- boDebug(401) << k_funcinfo << "unit " << id() << endl;
- if (advanceWork() != work()) {
-	if (work() == UnitBase::WorkAttack) {
-		// Unit is attacking. ATM it's moving to target.
-		// no need to move to the position of the unit...
-		// just check if unit is in range now.
-		if (!unit()->target()) {
-			boError() << k_funcinfo << "unit " << id() << " is in WorkAttack, but has NULL target!" << endl;
-			unit()->stopAttacking();
-			unit()->setMovingStatus(UnitBase::Standing);
-			return;
-		}
-		int range;
-		if (unit()->target()->isFlying()) {
-			range = unit()->maxAirWeaponRange();
-		} else {
-			range = unit()->maxLandWeaponRange();
-		}
-		if (unit()->inRange(range, unit()->target())) {
-			boDebug(401) << k_funcinfo << "unit " << id() << ": target is in range now" << endl;
-			unit()->setMovingStatus(UnitBase::Standing);
-			unit()->stopMoving();
-			return;
-		}
-		// TODO: make sure that target() hasn't moved!
-		// if it has moved also adjust waypoints
-	}
- } else if (pathInfo()->moveAttacking) {
+ if (pathInfo()->moveAttacking) {
 	// Attack any enemy units in range
 	// Don't check for enemies every time (if we don't have a target) because it
 	//  slows things down
-#ifndef CHECK_ENEMIES_ONLY_AT_WAYPOINT
-	if (unit()->target() || (advanceCallsCount % 10 == 0)) {
+	if (((UnitMoveOrderData*)unit()->currentOrderData())->target || (advanceCallsCount % 10 == 0)) {
 		if (attackEnemyUnitsInRangeWhileMoving()) {
 			return;
 		}
 	}
-#else
-	// Attack only if we already have target
-	if (unit()->target()) {
-		if (attackEnemyUnitsInRangeWhileMoving()) {
-			return;
-		}
-	}
-#endif
  }
 
- // x and y are center of the unit here
+ advanceMoveInternal3(advanceCallsCount);
+}
+
+
+// TODO: rename. maybe "actually move", "move or turn", "accelerate decelerate
+//       or turn" or something like that.
+//       -> here we actually "move", as in "set velocity" (moving is done
+//          _after_ the advance call in BosonCanvas).
+//          also we might need to calculate another part of the path, if path
+//          was calculated partially only
+void UnitMoverLand::advanceMoveInternal3(unsigned int advanceCallsCount)
+{
  bofixed x = unit()->centerX();
  bofixed y = unit()->centerY();
 
 
- //boDebug(401) << k_funcinfo << "unit " << id() << ": pos: (" << x << "; "<< y << ")" << endl;
- // If we're close to destination, decelerate, otherwise  accelerate
+ // If we're close to destination, decelerate, otherwise accelerate
  // TODO: we should also slow down when turning at pathpoint.
  // TODO: support range != 0
  bofixed oldspeed = unit()->speed();
@@ -525,6 +554,7 @@ void UnitMoverLand::advanceMoveInternal(unsigned int advanceCallsCount)
  bofixed dist = unit()->speed();
  BoVector2Fixed pp;
 
+
  // We move through the pathpoints, until we've passed dist distance
  while (dist > 0) {
 	// Check if we have any pathpoints left
@@ -532,15 +562,22 @@ void UnitMoverLand::advanceMoveInternal(unsigned int advanceCallsCount)
 		if (pathInfo()->result == BosonPath::OutOfRange) {
 			// New pathfinding query has to made, because last returned path was
 			//  only partial.
-			if (!newPath()) {
+			if (!calculateNewPath()) {
 				// Probably no path could be found
 				// Unit will stop moving during the next advance call (because we want
 				//  to move as much as we already have in this loop)
+				if (xspeed == 0 && yspeed == 0) {
+					stopMoving(false);  // TODO: is false correct here?
+					return;
+				}
 				break;
 			}
 		} else {
 			// TODO: rotate a bit randomly
-			// TODO: support multiple waypoints
+			if (xspeed == 0 && yspeed == 0) {
+				stopMoving(true);
+				return;
+			}
 			break;
 		}
 	}
@@ -575,7 +612,7 @@ void UnitMoverLand::advanceMoveInternal(unsigned int advanceCallsCount)
 	// Make sure it's possible to go to the pathpoint
 	if (!canGoToCurrentPathPoint(x + xspeed, y + yspeed)) {
 		// Gotta find another path
-		if (!newPath()) {
+		if (!calculateNewPath()) {
 			// Probably no path could be found
 			// Unit will stop moving during the next advance call (because we want
 			//  to move as much as we already have in this loop)
@@ -603,6 +640,7 @@ void UnitMoverLand::advanceMoveInternal(unsigned int advanceCallsCount)
 	}
  }
 
+
  // If the unit didn't move, we can just return now (this is valid)
  if ((xspeed == 0) && (yspeed == 0)) {
 	return;
@@ -614,24 +652,32 @@ void UnitMoverLand::advanceMoveInternal(unsigned int advanceCallsCount)
 
  //avoidance();
 
- turnTo();
+ bool turning = turnTo();
 
  // If we just started moving and now want to turn, then set velocity back to 0
- if ((advanceWork() == UnitBase::WorkTurn) && (oldspeed == 0)) {
+ if (turning && (oldspeed == 0)) {
 	unit()->setVelocity(0, 0, 0);
  }
 }
 
-// AB: WARNING crushing is currently disabled!
-//
-// see #warning below
-void UnitMoverLand::advanceMoveCheck()
+
+// TODO: rename. maybe "check for waiting" or "check fo blocked path" or so.
+// -> this handles "waiting" (when path is blocked)
+void UnitMoverLand::advanceMoveInternal4(unsigned int advanceCallsCount)
 {
  PROFILE_METHOD;
  //boDebug(401) << k_funcinfo << endl;
 
+ if (!unit()->currentOrder() || (unit()->currentOrder()->type() != UnitOrder::Move && unit()->currentOrder()->type() != UnitOrder::MoveToUnit)) {
+	// We might be e.g. turning
+	// TODO: before turning, we have possibly already moved, so we should still
+	//  check our new pos here
+	// TODO: also consider e.g. MoveToUnit order
+	return;
+ }
+
  // Take special action if path is (was) blocked and we're waiting
- if (pathInfo()->waiting) {
+ if (pathInfo()->waiting != 0) {
 	// We try to move every 5 advance calls
 	if (pathInfo()->waiting % 5 == 0) {
 		// Try to move again
@@ -639,15 +685,14 @@ void UnitMoverLand::advanceMoveCheck()
 			// Obstacle is still there. Continue waiting
 			if (pathInfo()->waiting >= 600) {
 				// Enough of waiting (30 secs). Give up.
-				unit()->stopMoving();
-				unit()->setWork(UnitBase::WorkIdle);
+				stopMoving(false);
 				return;
 			} else if (pathInfo()->waiting % (20 + QMIN(pathInfo()->pathrecalced * 20, 80)) == 0) {
 				// First wait 20 adv. calls (1 sec) before recalculating path, then 40
 				//  calls, then 60 etc, but never more than 100 calls.
 				boDebug(401) << k_funcinfo << "unit " << id() << ": Recalcing path, waiting: " << pathInfo()->waiting <<
 						"; pathrecalced: " << pathInfo()->pathrecalced << endl;
-				newPath();
+				calculateNewPath();
 				pathInfo()->pathrecalced++;
 				pathInfo()->waiting = 0;
 				// New path will be used next advance call
@@ -670,23 +715,13 @@ void UnitMoverLand::advanceMoveCheck()
 	}
  }
 
- // Make sure unit is one canvas
- UnitMover::advanceMoveCheck();
+ advanceMoveInternal5(advanceCallsCount);
+}
 
- if (unit()->pathPointCount() == 0) {
-	// This is allowed and means that unit will stop after this this advance call
-	return;
- }
-
- if (unit()->xVelocity() == 0 && unit()->yVelocity() == 0) {
-	// Probably unit stopped to attack other units
-	return;
- }
-
- //boDebug(401) << k_funcinfo << "unit " << id() << endl;
-
- // Crushing & waiting
-#warning What the hell does crushing do in advanceMove_Check_() ??
+void UnitMoverLand::advanceMoveDoCrushing(unsigned int advanceCallsCount)
+{
+ // AB: FIXME: this really is no moving code. it does not belong in this class
+ // at all.
 #define CRUSHING 0
 #if CRUSHING
  bool wait = false;
@@ -704,7 +739,7 @@ void UnitMoverLand::advanceMoveCheck()
 			continue;
 		}
 		// Make sure we actually can crush this unit
-		if (u->maxHealth() > unitProperties()->crushDamage()) {
+		if (!unit()->canCrush(u)) {
 			boDebug(401) << k_funcinfo << id() << ": Colliding with uncrushable unit " << u->id() << endl;
 			// Whoops. Now what?
 			wait = true;
@@ -716,18 +751,28 @@ void UnitMoverLand::advanceMoveCheck()
 		canvas()->destroyUnit(u);
 	}
  }
-#endif
- /*if (wait) {
-	setVelocity(0, 0, 0);
-	setMovingStatus(UnitBase::Waiting);
-	setSpeed(0);
- }*/
+#endif // CRUSHING
+#undef CRUSHING
+}
+
+// TODO: rename.
+void UnitMoverLand::advanceMoveInternal5(unsigned int advanceCallsCount)
+{
+ if (unit()->pathPointCount() == 0) {
+	// This is allowed and means that unit will stop after this advance call
+	return;
+ }
+
+ if (unit()->xVelocity() == 0 && unit()->yVelocity() == 0) {
+	// Probably unit stopped to attack other units
+	return;
+ }
 
  // Check if we need to wait
  // Find the next cell we'll be on
  if (mNextWaypointIntersections->count() == 0) {
 	boWarning() << k_funcinfo << unit()->id() << ": mNextWaypointIntersections is empty" << endl;
-	unit()->stopMoving();
+	stopMoving(false);
 	return;
  }
 
@@ -770,8 +815,61 @@ void UnitMoverLand::advanceMoveCheck()
 
  pathInfo()->waiting = 0;
  pathInfo()->pathrecalced = 0;
+}
 
- //boDebug(401) << k_funcinfo << "unit " << id() << ": done" << endl;
+// AB: WARNING crushing is currently disabled!
+//     -> it does NOT belong into this method!
+//
+// AB: advanceMoveCheck() was meant to _check_ whether the move is valid, i.e.
+//     check if the calculated velocity or so may cause the unit to go off the
+//     map or collide with a different unit.
+//     however atm it is heavily used to actually do a lot of calculations,
+//     which is not intended at all.
+//     this should be fixed.
+void UnitMoverLand::advanceMoveCheck()
+{
+ PROFILE_METHOD;
+ BO_CHECK_NULL_RET(unit());
+
+ if (unit()->xVelocity() == 0 && unit()->yVelocity() == 0) {
+	return;
+ }
+
+ // Make sure unit is on canvas
+ UnitMover::advanceMoveCheck();
+
+ if (!unit()->currentOrder() || (unit()->currentOrder()->type() != UnitOrder::Move && unit()->currentOrder()->type() != UnitOrder::MoveToUnit)) {
+	// apparently the velocity was set to something != 0 although it should
+	// not. revert.
+	unit()->setVelocity(0, 0, 0);
+	return;
+ }
+
+// TODO .. rewrite this. should check for collisions on position + velocity
+//         --> if collisions occur: check if they are about to be crushed. if
+//             not: don't move there. stop moving completely.
+#if 0
+ // Make sure the next cell is free of any obstacles
+ if (unit()->cellOccupied(mNextCellX, mNextCellY)) {
+	// Gotta wait
+	unit()->setVelocity(0, 0, 0);
+	unit()->setSpeed(0);
+	stopMoving(false);
+	return;
+ }
+#endif
+
+
+
+// AB: crushing does NOT belong here. but it MUST be after advanceMoveCheck() -
+//     i.e. at a pointer where the unit _definitely_ goes by the velocities it
+//     has been set to.
+#warning What the hell does crushing do in advanceMove_Check_() ??
+#define CRUSHING 0
+#if CRUSHING
+ advanceMoveDoCrushing(advanceCallsCount);
+#endif // CRUSHING
+#undef CRUSHING
 }
 
 void UnitMoverLand::currentPathPointChanged(int unitx, int unity)
@@ -815,7 +913,7 @@ int UnitMoverLand::selectNextPathPoint(int xpos, int ypos)
 		}
 	}
 	if (cango) {
-		// skip previous waypoint (go directly to this one)
+		// skip previous pathpoint (go directly to this one)
 		unit()->pathPointDone();
 		currentPathPointChanged(xpos, ypos);
 		skipped++;
@@ -833,8 +931,10 @@ void UnitMoverLand::avoidance()
  BoVector2Fixed toRight(toRight3.x(), -toRight3.y());
  bofixed avoidstrength = 0;
  // Find all units which are near us
- BoRect2Fixed rect(unit()->centerX() - unit()->width() - unit()->speed() * 20 - 3, unit()->centerY() - unit()->width() - unit()->speed() * 20 - 3,
-		unit()->centerX() + unit()->width() + unit()->speed() * 20 + 3, unit()->centerY() + unit()->width() + unit()->speed() * 20 + 3);
+ BoRect2Fixed rect(unit()->x() - unit()->speed() * 40 - 1,
+		unit()->y() - unit()->speed() * 40 - 1,
+		unit()->x() + unit()->width() + unit()->speed() * 40 + 1,
+		unit()->y() + unit()->height() + unit()->speed() * 40 + 1);
  BoItemList* items = canvas()->collisions()->collisionsAtCells(rect, unit(), false);
  // Go through the units
  for (BoItemList::ConstIterator it = items->begin(); it != items->end(); ++it) {
@@ -965,14 +1065,21 @@ bofixed UnitMoverLand::moveTowardsPoint(const BoVector2Fixed& p, bofixed x, bofi
  }
 }
 
-bool UnitMoverLand::newPath()
+bool UnitMoverLand::calculateNewPath()
 {
- BosonProfiler profiler("newPath");
+ BosonProfiler profiler("calculateNewPath");
  boDebug(401) << k_funcinfo << "unit " << id() << endl;
 
  // Update our start position
  pathInfo()->start.set(unit()->centerX(), unit()->centerY());
 
+ if (unit()->currentOrder()->type() == UnitOrder::MoveToUnit) {
+	// Update destination
+	const BoVector2Fixed& targetpos = ((UnitMoveToUnitOrder*)unit()->currentOrder())->target()->center();
+	pathInfo()->dest = targetpos;
+	((UnitMoveToUnitOrderData*)unit()->currentOrderData())->lastTargetPos = targetpos;
+ }
+
  // Clear previous pathpoints
  unit()->clearPathPoints();
 
@@ -999,6 +1106,10 @@ bool UnitMoverLand::newPath()
  mLastCellY = -1;
  mNextWaypointIntersections = &mCellIntersectionTable[5][5];
 
+ // AB: FIXME: do we need a unit()->setMovingStatus() here?
+ //            -> it's pretty dumb to have movingStatus() == MustSearchPath at
+ //               this point
+
  return true;
 }
 
@@ -1069,6 +1180,9 @@ void UnitMoverFlying::advanceIdle()
 
 void UnitMoverFlying::flyInCircle()
 {
+ if (unit()->maxSpeed() == 0) {
+	return;
+ }
  bofixed speedfactor = unit()->speed() / unit()->maxSpeed();
  if (speedfactor < 0.7) {
 	unit()->accelerate();
@@ -1078,7 +1192,7 @@ void UnitMoverFlying::flyInCircle()
 
  // Flying units need to keep flying
  // TODO: choose which way to turn
- bofixed newrot = unit()->rotation() + unit()->maxSpeed() * 20;
+ bofixed newrot = unit()->rotation() + unitProperties()->rotationSpeed();
  if (newrot > 360) {
 	newrot -= 360;
  }
@@ -1089,8 +1203,13 @@ void UnitMoverFlying::flyInCircle()
  // Don't go off the map
  if (unit()->x() < 0.5 || unit()->y() < 0.5 ||
 		unit()->x() > bofixed(canvas()->mapWidth()) - unit()->width() - 0.5 || unit()->y() > bofixed(canvas()->mapHeight()) - unit()->height() - 0.5) {
-	unit()->move(QMIN(QMAX(unit()->x(), bofixed(1)), bofixed(canvas()->mapWidth()) - unit()->width() - 1),
-			QMIN(QMAX(unit()->y(), bofixed(1)), bofixed(canvas()->mapHeight()) - unit()->height() - 1), unit()->z());
+	bofixed x = unit()->x();
+	bofixed y = unit()->y();
+	x = QMAX(x, bofixed(1));
+	y = QMAX(y, bofixed(1));
+	x = QMIN(x, bofixed(canvas()->mapWidth()) - unit()->width() - 1);
+	y = QMIN(y, bofixed(canvas()->mapHeight()) - unit()->height() - 1);
+	unit()->move(x, y, unit()->z());
  }
 
  bofixed groundz = canvas()->heightAtPoint(unit()->centerX() + velo.x(), unit()->centerY() + velo.y());
@@ -1126,7 +1245,7 @@ void UnitMoverFlying::advanceMoveInternal(unsigned int advanceCallsCount)
  if (unit()->maxSpeed() == 0) {
 	// If unit's max speed is 0, it cannot move
 	boError(401) << k_funcinfo << "unit " << id() << ": maxspeed == 0" << endl;
-	unit()->stopMoving();
+	stopMoving(false);
 	unit()->setMovingStatus(UnitBase::Standing);
 	return;
  }
@@ -1135,9 +1254,14 @@ void UnitMoverFlying::advanceMoveInternal(unsigned int advanceCallsCount)
 
 
  // Calculate velocity
- // Missile always moves towards it's target
- BoVector3Fixed targetpos(unit()->currentWaypoint().x(), unit()->currentWaypoint().y(), 0);
+ BoVector3Fixed targetpos(unit()->pathInfo()->dest.x(), unit()->pathInfo()->dest.y(), 0);
+ if (unit()->currentOrder()->type() == UnitOrder::MoveToUnit) {
+	const BoVector2Fixed& targetunitpos = ((UnitMoveToUnitOrder*)unit()->currentOrder())->target()->center();
+	targetpos.set(targetunitpos.x(), targetunitpos.y(), 0);
+ }
  targetpos.setZ(canvas()->heightAtPoint(targetpos.x(), targetpos.y()) + 3);
+#warning TODO!!!
+#if 0
  if (advanceWork() != work()) {
 	if (work() == UnitBase::WorkAttack) {
 		// Unit is attacking. ATM it's moving to target.
@@ -1165,11 +1289,13 @@ void UnitMoverFlying::advanceMoveInternal(unsigned int advanceCallsCount)
 			targetpos.setZ(targetpos.z() + 3);
 		}
 	}
- } else if (pathInfo()->moveAttacking) {
+ } else
+#endif
+if (pathInfo()->moveAttacking) {
 	// Attack any enemy units in range
 	// Don't check for enemies every time (if we don't have a target) because it
 	//  slows things down
-	if (unit()->target() || (advanceCallsCount % 10 == 0)) {
+	if (((UnitMoveOrderData*)unit()->currentOrderData())->target || (advanceCallsCount % 10 == 0)) {
 		attackEnemyUnitsInRangeWhileMoving();
 	}
  }
@@ -1187,8 +1313,8 @@ void UnitMoverFlying::advanceMoveInternal(unsigned int advanceCallsCount)
  BoVector3Fixed totarget(targetpos.x() - x, targetpos.y() - y, targetpos.z() - z);
  bofixed totargetlen = totarget.length();
  // We need check this here to avoid division by 0 later
- if (totargetlen <= unit()->speed()) {
-	unit()->stopMoving();
+ if (totargetlen <= pathInfo()->range || totargetlen <= unit()->speed()) {
+	stopMoving(true);
 	return;
  }
  // Normalize totarget. totarget vector now shows direction to target
@@ -1210,7 +1336,7 @@ void UnitMoverFlying::advanceMoveInternal(unsigned int advanceCallsCount)
 	turncw = !turncw;
  }
 
- const bofixed maxturningspeed = 2;
+ const bofixed maxturningspeed = unitProperties()->rotationSpeed();
  if (rotationdelta <= maxturningspeed) {
 	newrotation = wantedrotation;
  } else {
@@ -1297,3 +1423,4 @@ void UnitMoverFlying::advanceMoveInternal(unsigned int advanceCallsCount)
 
  unit()->setMovingStatus(UnitBase::Moving);
 }
+
