@@ -1,7 +1,7 @@
 /*
     This file is part of the Boson game
-    Copyright (C) 2001-2005 Andreas Beckermann (b_mann@gmx.de)
-    Copyright (C) 2001-2005 Rivo Laks (rivolaks@hot.ee)
+    Copyright (C) 2001-2006 Andreas Beckermann (b_mann@gmx.de)
+    Copyright (C) 2001-2006 Rivo Laks (rivolaks@hot.ee)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,34 +24,38 @@
 #include "../bomemory/bodummymemory.h"
 #include "../no_player.h"
 #include "bosoncanvasrenderer.h"
-#include "../bosoncanvas.h"
-#include "../bosonmap.h"
-#include "../bosonmodel.h"
-#include "../speciestheme.h"
+#include "../gameengine/bosoncanvas.h"
+#include "../gameengine/bosonmap.h"
+#include "../modelrendering/bosonmodel.h"
+#include "../gameengine/speciestheme.h"
 #include "../speciesdata.h"
-#include "../bosongroundtheme.h"
-#include "../playerio.h"
-#include "../unitproperties.h"
+#include "../gameengine/bosongroundtheme.h"
+#include "../gameengine/playerio.h"
+#include "../gameengine/unitproperties.h"
 #include "../bosonconfig.h"
 #include "../bosonprofiling.h"
-#include "../items/bosonshot.h"
-#include "../items/bosonitemrenderer.h"
-#include "../bosonweapon.h"
-#include "../unit.h"
-#include "../bosoneffectmanager.h"
-#include "../bosoneffectproperties.h"
-#include "../bosoneffect.h"
+#include "../gameengine/bosonshot.h"
+#include "bosonitemrenderer.h"
+#include "../gameengine/bosonweapon.h"
+#include "../gameengine/unit.h"
+#include "bosoneffectmanager.h"
+#include "bosoneffectproperties.h"
+#include "bosoneffect.h"
 #include "../bowaterrenderer.h"
 #include "../bocamera.h"
-#include "../boitemlist.h"
+#include "../gameengine/boitemlist.h"
 #include "../bosonviewdata.h"
+#include "../gameengine/bogroundquadtreenode.h"
 #include "bodebug.h"
 
+#include "../botexture.h"
+
 #include <klocale.h>
 
 #include <qtimer.h>
 #include <qvaluelist.h>
 #include <qptrdict.h>
+#include <qptrlist.h>
 #include <qdom.h>
 
 
@@ -124,6 +128,7 @@ void BosonItemEffects::updateEffectsRotation()
 
 
 
+
 class BosonUfoCanvasWidgetPrivate
 {
 public:
@@ -136,6 +141,8 @@ public:
 		mCanvas = 0;
 
 		mEffectManager = 0;
+
+		mGroundQuadTree = 0;
 	}
 	const BoGLMatrices* mGameGLMatrices;
 	BosonCanvasRenderer* mCanvasRenderer;
@@ -146,6 +153,8 @@ public:
 	QPtrList<BosonEffect> mEffects;
 
 	BosonEffectManager* mEffectManager;
+
+	BoGroundQuadTreeNode* mGroundQuadTree;
 };
 
 BosonUfoCanvasWidget::BosonUfoCanvasWidget()
@@ -160,29 +169,52 @@ BosonUfoCanvasWidget::BosonUfoCanvasWidget()
  d->mCanvasRenderer = new BosonCanvasRenderer();
  d->mCanvasRenderer->initGL();
 
- if (!boViewData) {
-	BO_NULL_ERROR(boViewData);
-	return;
- }
- connect(boViewData, SIGNAL(signalItemContainerAdded(BosonItemContainer*)),
-		this, SLOT(slotAddItemContainerData(BosonItemContainer*)));
- connect(boViewData, SIGNAL(signalItemContainerAboutToBeRemoved(BosonItemContainer*)),
-		this, SLOT(slotRemoveItemContainerData(BosonItemContainer*)));
- if (boViewData->allItemContainers().count() > 0) {
-	boError() << k_funcinfo << "itemcontainer list should be empty at this point" << endl;
- }
+ connect(this, SIGNAL(signalWidgetResized()), d->mCanvasRenderer, SLOT(slotWidgetResized()));
 }
 
 BosonUfoCanvasWidget::~BosonUfoCanvasWidget()
 {
- boDebug() << k_funcinfo << endl;
  quitGame();
+ delete d->mGroundQuadTree;
  delete d->mCanvasRenderer;
  delete d->mEffectManager;
  BosonEffectPropertiesManager::deleteStatic();
  delete d;
+}
 
- boDebug() << k_funcinfo << "donee" << endl;
+bool BosonUfoCanvasWidget::initializeItems()
+{
+ if (!boViewData) {
+	BO_NULL_ERROR(boViewData);
+	return false;
+ }
+ disconnect(boViewData, 0, this, 0);
+ connect(boViewData, SIGNAL(signalItemContainerAdded(BosonItemContainer*)),
+		this, SLOT(slotAddItemContainerData(BosonItemContainer*)));
+ connect(boViewData, SIGNAL(signalItemContainerAboutToBeRemoved(BosonItemContainer*)),
+		this, SLOT(slotRemoveItemContainerData(BosonItemContainer*)));
+
+ for (QPtrListIterator<BosonItemContainer> it(boViewData->allItemContainers()); it.current(); ++it) {
+	if (it.current()->effects()) {
+		boError() << k_funcinfo << "container already has effects. double initializing!" << endl;
+		return false;
+	}
+	if (it.current()->itemRenderer()) {
+		boError() << k_funcinfo << "container already has an item renderer. double initializing!" << endl;
+		return false;
+	}
+ }
+ for (QPtrListIterator<BosonItemContainer> it(boViewData->allItemContainers()); it.current(); ++it) {
+	slotAddItemContainerData(it.current());
+ }
+
+ if (!d->mCanvas) {
+	BO_NULL_ERROR(d->mCanvas);
+	return false;
+ }
+ initItemEffects();
+
+ return true;
 }
 
 void BosonUfoCanvasWidget::setGameGLMatrices(const BoGLMatrices* m)
@@ -199,6 +231,7 @@ void BosonUfoCanvasWidget::setCamera(BoGameCamera* c)
 void BosonUfoCanvasWidget::setLocalPlayerIO(PlayerIO* io)
 {
  d->mLocalPlayerIO = io;
+ d->mCanvasRenderer->setLocalPlayerIO(d->mLocalPlayerIO);
 }
 
 void BosonUfoCanvasWidget::setCanvas(const BosonCanvas* canvas)
@@ -206,7 +239,10 @@ void BosonUfoCanvasWidget::setCanvas(const BosonCanvas* canvas)
  if (d->mCanvas) {
 	disconnect(d->mCanvas, 0, this, 0);
  }
+ delete d->mGroundQuadTree;
+ d->mGroundQuadTree = 0;
  d->mCanvas = canvas;
+ d->mCanvasRenderer->setCanvas(d->mCanvas);
  if (d->mCanvas) {
 	connect(d->mCanvas, SIGNAL(signalShotFired(BosonShot*, BosonWeapon*)),
 		this, SLOT(slotShotFired(BosonShot*, BosonWeapon*)));
@@ -216,15 +252,36 @@ void BosonUfoCanvasWidget::setCanvas(const BosonCanvas* canvas)
 		this, SLOT(slotUnitDestroyed(Unit*)));
 	connect(d->mCanvas, SIGNAL(signalFragmentCreated(BosonShotFragment*)),
 		this, SLOT(slotFragmentCreated(BosonShotFragment*)));
+	d->mGroundQuadTree = BoGroundQuadTreeNode::createTree(d->mCanvas->mapWidth(), d->mCanvas->mapHeight());
 
-	for (BoItemList::ConstIterator it = d->mCanvas->allItems()->begin(); it != d->mCanvas->allItems()->end(); ++it) {
-		if (RTTI::isUnit((*it)->rtti())) {
-			Unit* u = (Unit*)*it;
-			if (u->isFacility()) {
-				Facility* f = (Facility*)u;
-				if (f->isConstructionComplete()) {
-					addFacilityConstructedEffects(u);
-				}
+	const BosonMap* map = d->mCanvas->map();
+
+	// AB: atm we need a non-const map pointer for registering :-(
+	//     -> however we don't really modify the map, so it is safe to use
+	//        nonConstMap here.
+	BosonMap* nonConstMap = d->mCanvas->map();
+	nonConstMap->registerQuadTree(d->mGroundQuadTree);
+
+	BosonProfiler treeProf("Initialize tree");
+	d->mGroundQuadTree->cellTextureChanged(map, 0, 0, map->width() - 1, map->height() - 1);
+	d->mGroundQuadTree->cellHeightChanged(map, 0, 0, map->width() - 1, map->height() - 1);
+	treeProf.pop();
+ }
+}
+
+void BosonUfoCanvasWidget::initItemEffects()
+{
+ if (d->mEffects.count() > 0) {
+	boDebug() << k_funcinfo << "effects already initialized. most likely loaded from xml - we are loading a game, no initializing required." << endl;
+	return;
+ }
+ for (BoItemList::ConstIterator it = d->mCanvas->allItems()->begin(); it != d->mCanvas->allItems()->end(); ++it) {
+	if (RTTI::isUnit((*it)->rtti())) {
+		Unit* u = (Unit*)*it;
+		if (u->isFacility()) {
+			UnitConstruction* c = u->construction();
+			if (c->isConstructionComplete()) {
+				addFacilityConstructedEffects(u);
 			}
 		}
 	}
@@ -238,6 +295,8 @@ void BosonUfoCanvasWidget::setParticlesDirty(bool dirty)
 
 void BosonUfoCanvasWidget::quitGame()
 {
+ delete d->mGroundQuadTree;
+ d->mGroundQuadTree = 0;
  d->mCanvasRenderer->reset();
 
  while (!d->mEffects.isEmpty()) {
@@ -257,6 +316,9 @@ void BosonUfoCanvasWidget::quitGame()
 	}
 	delete e;
  }
+
+ disconnect(boViewData, SIGNAL(signalItemContainerAdded(BosonItemContainer*)),
+		this, 0);
 }
 
 void BosonUfoCanvasWidget::addEffect(BosonEffect* e)
@@ -271,6 +333,25 @@ void BosonUfoCanvasWidget::addEffects(const QPtrList<BosonEffect>& effects)
  }
 }
 
+void BosonUfoCanvasWidget::createEffect(unsigned int id, const BoVector3Fixed& pos, bofixed zrot)
+{
+ addEffects(d->mEffectManager->newEffects(id, pos, zrot));
+}
+
+void BosonUfoCanvasWidget::createAttachedEffect(int itemid, unsigned int effectid, BoVector3Fixed offset, bofixed zrot)
+{
+ BosonItem* item = d->mCanvas->findItem(itemid);
+ if (!item) {
+	boError() << k_funcinfo << "No item with id " << itemid << endl;
+	return;
+ }
+ BosonItemContainer* c = boViewData->itemContainer(item);
+ BO_CHECK_NULL_RET(c);
+ BosonItemEffects* effects = c->effects();
+ BO_CHECK_NULL_RET(effects);
+ effects->setEffects(d->mEffectManager->newEffects(effectid, offset, zrot), &d->mEffects);
+}
+
 void BosonUfoCanvasWidget::slotAdvance(unsigned int advanceCallsCount, bool advanceFlag)
 {
  Q_UNUSED(advanceFlag);
@@ -301,6 +382,10 @@ void BosonUfoCanvasWidget::advanceEffects(float elapsed)
  QPtrList<BosonEffect> removeEffects;
  for (QPtrListIterator<BosonItemContainer> it(boViewData->allItemContainers()); it.current(); ++it) {
 	BosonItemEffects* e = it.current()->effects();
+	if (!e) {
+		BO_NULL_ERROR(e);
+		return;
+	}
 	if (!e->item()) {
 		continue;
 	}
@@ -491,7 +576,7 @@ void BosonUfoCanvasWidget::slotShotHit(BosonShot* shot)
 	d->mEffectManager->loadWeaponType(shot->properties());
 	switch (shot->type()) {
 		case BosonShot::Bullet:
-			effects->setEffects(d->mEffectManager->newFlyEffects(shot->properties(), pos, 0), &d->mEffects);
+			effects->updateEffectsPosition();
 			break;
 		case BosonShot::Rocket:
 			break;
@@ -614,9 +699,15 @@ void BosonUfoCanvasWidget::slotAddItemContainerData(BosonItemContainer* c)
 	return;
  }
 
+ if (c->effects()) {
+	boWarning() << k_funcinfo << "container effects non-NULL. probably memory leak" << endl;
+ }
  BosonItemEffects* effects = new BosonItemEffects(item);
  c->setEffects(effects);
 
+ if (c->itemRenderer()) {
+	boWarning() << k_funcinfo << "container itemrenderer non-NULL. probably memory leak" << endl;
+ }
  BosonItemRenderer* itemRenderer = createItemRendererFor(c);
  if (!itemRenderer) {
 	boWarning() << k_funcinfo << "unable to initialize item renderer for item " << item->id() << endl;
@@ -638,7 +729,9 @@ void BosonUfoCanvasWidget::slotAddItemContainerData(BosonItemContainer* c)
 	BosonShot* shot = (BosonShot*)item;
 	switch (shot->type()) {
 		case BosonShot::Bullet:
-			break;
+			// Fall through
+		case BosonShot::Missile:
+			// Fall through
 		case BosonShot::Rocket:
 		{
 			d->mEffectManager->loadWeaponType(shot->properties());
@@ -654,13 +747,6 @@ void BosonUfoCanvasWidget::slotAddItemContainerData(BosonItemContainer* c)
 			break;
 		case BosonShot::Fragment:
 			break;
-		case BosonShot::Missile:
-		{
-			d->mEffectManager->loadWeaponType(shot->properties());
-			BoVector3Fixed pos(shot->x(), shot->y(), shot->z());
-			effects->setEffects(d->mEffectManager->newFlyEffects(shot->properties(), pos, 0.0), &d->mEffects);
-			break;
-		}
 		default:
 			break;
 	}
@@ -681,6 +767,314 @@ void BosonUfoCanvasWidget::slotRemoveItemContainerData(BosonItemContainer* c)
  delete itemRenderer;
 }
 
+QValueList<BosonItem*> BosonUfoCanvasWidget::itemsAtWidgetRect(const QRect& widgetRect) const
+{
+ return emulatePickItems(widgetRect);
+}
+
+QValueList<Unit*> BosonUfoCanvasWidget::unitsAtWidgetRect(const QRect& widgetRect) const
+{
+ QValueList<Unit*> units;
+ QValueList<BosonItem*> items = itemsAtWidgetRect(widgetRect);
+ for (QValueList<BosonItem*>::iterator it = items.begin(); it != items.end(); ++it) {
+	if (RTTI::isUnit((*it)->rtti())) {
+		units.append((Unit*)*it);
+	}
+ }
+ return units;
+}
+
+Unit* BosonUfoCanvasWidget::unitAtWidgetPos(const QPoint& widgetPos) const
+{
+ QValueList<Unit*> units = unitsAtWidgetRect(QRect(widgetPos, QSize(1, 1)));
+ if (units.count() == 0) {
+	return 0;
+ }
+
+ // TODO: select the unit that is closest to the player
+
+ return units[0];
+}
+
+QValueList<BosonItem*> BosonUfoCanvasWidget::emulatePickItems(const QRect& pickRect) const
+{
+ return d->mCanvasRenderer->emulatePickItems(pickRect);
+}
+
+/**
+ * @shortHelper clas for @ref emulatePickGroundPos
+ *
+ * @ref emulatePickGroundPos needs a higher precision than our cell-based
+ * quadtree provides, so we divide leafs even further, using this class.
+ *
+ * We cannot use @ref BoRect3Float, as there we can have min/max values for z
+ * only, but we need the correct z value for every corner.
+ **/
+class EmulatePickRect {
+public:
+	EmulatePickRect()
+	{
+	}
+	EmulatePickRect(const EmulatePickRect& n)
+	{
+		*this = n;
+	}
+	EmulatePickRect(const BoVector3Float& tl, const BoVector3Float& tr, const BoVector3Float& bl, const BoVector3Float& br)
+	{
+		topLeft = tl;
+		topRight = tr;
+		bottomLeft = bl;
+		bottomRight = br;
+	}
+	EmulatePickRect& operator=(const EmulatePickRect& n)
+	{
+		topLeft = n.topLeft;
+		topRight = n.topRight;
+		bottomLeft = n.bottomLeft;
+		bottomRight = n.bottomRight;
+		return *this;
+	}
+
+	BoVector3Float center() const
+	{
+		// AB: we assume that topLeft.x() == bottomLeft().x() (same
+		// about Right/Top/Bottom)
+		const float x = (topLeft.x() + topRight.x()) / 2.0f;
+		const float y = (topLeft.y() + bottomLeft.y()) / 2.0f;
+		const float z = (topLeft.z() + topRight.z() + bottomLeft.z() + bottomRight.z()) / 4.0f;
+		return BoVector3Float(x, y, z);
+	}
+	float radius() const
+	{
+		BoVector3Float c = center();
+		float z1 = (c - topRight).length();
+		float z2 = (c - topLeft).length();
+		float z3 = (c - bottomRight).length();
+		float z4 = (c - bottomLeft).length();
+		float r = QMAX(z1, z2);
+		r = QMAX(r, z3);
+		r = QMAX(r, z4);
+		return r;
+	}
+
+	BoRect3Float boundingBox() const
+	{
+		float minZ = topLeft.z();
+		float maxZ = minZ;
+		minZ = QMIN(minZ, topRight.z());
+		maxZ = QMAX(maxZ, topRight.z());
+		minZ = QMIN(minZ, bottomRight.z());
+		maxZ = QMAX(maxZ, bottomRight.z());
+		minZ = QMIN(minZ, bottomLeft.z());
+		maxZ = QMAX(maxZ, bottomLeft.z());
+		return BoRect3Float(topLeft.x(), topLeft.y(), minZ, bottomRight.x(), bottomRight.y(), maxZ);
+	}
+	BoVector3Float topLeft;
+	BoVector3Float topRight;
+	BoVector3Float bottomLeft;
+	BoVector3Float bottomRight;
+};
+
+/*
+ * AB: the code sucks. It is pretty complex and should be simplified.
+ *
+ * However it works and as long as it does, I don't intend to do so.
+ */
+bool BosonUfoCanvasWidget::emulatePickGroundPos(const QPoint& pickPos, BoVector3Float* worldPos) const
+{
+ PROFILE_METHOD
+ if (!worldPos) {
+	BO_NULL_ERROR(worldPos);
+	return false;
+ }
+ if (!d->mCanvas) {
+	BO_NULL_ERROR(d->mCanvas);
+	return false;
+ }
+ const BosonMap* map = d->mCanvas->map();
+ if (!map) {
+	BO_NULL_ERROR(map);
+	return false;
+ }
+ const float pickSize = 0.1f;
+ BoRect2Float pickRect((float)pickPos.x() - pickSize, (float)pickPos.y() - pickSize, (float)pickPos.x() + pickSize, (float)pickPos.y() + pickSize);
+ BoFrustum viewFrustum;
+ viewFrustum.loadPickViewFrustum(pickRect, d->mGameGLMatrices->viewport(), d->mGameGLMatrices->modelviewMatrix(), d->mGameGLMatrices->projectionMatrix());
+ BoVector3Float pos;
+
+ QValueVector<BoGroundQuadTreeNode*> retNodes;
+ QValueVector<BoGroundQuadTreeNode*> nodes;
+ nodes.append(d->mGroundQuadTree);
+ int visibleDepth = 0;
+ while (!nodes.isEmpty()) {
+	QValueVector<BoGroundQuadTreeNode*> nextNodes;
+	nextNodes.reserve(nodes.count() * 4);
+
+	QValueVector<BoGroundQuadTreeNode*> nextRetNodes;
+	nextRetNodes.reserve(nodes.count());
+
+	QValueVector<BoGroundQuadTreeNode*>::iterator it;
+	for (it = nodes.begin(); it != nodes.end(); ++it) {
+		BoRect3Float r = (*it)->groundBoundingBox();
+
+#define USE_BOX_NOT_SPHERE 1
+#if USE_BOX_NOT_SPHERE
+		if (!viewFrustum.boxInFrustum(r)) {
+			continue;
+		}
+#else
+		BoVector3Float center;
+		center.setX((r.left() + r.right()) / 2.0f);
+		center.setY((r.top() + r.bottom()) / 2.0f);
+		center.setZ((r.front() + r.back()) / 2.0f);
+		BoVector3Float v = center + BoVector3Float(r.width() / 2.0f, r.height() / 2.0f, r.depth() / 2.0f);
+		float radius = (center - v).length();
+		if (!viewFrustum.sphereInFrustum(center, radius)) {
+			continue;
+		}
+#endif
+#undef USE_BOX_NOT_SPHERE
+
+		nextRetNodes.append(*it);
+
+		BoQuadTreeNode* children[4];
+		(*it)->getChildren(children);
+		for (int j = 0; j < 4; j++) {
+			if (children[j]) {
+				nextNodes.append((BoGroundQuadTreeNode*)children[j]);
+			}
+		}
+	}
+	if (nextRetNodes.count() > 0) {
+		retNodes = nextRetNodes;
+	}
+	nodes = nextNodes;
+	visibleDepth++;
+ }
+
+ if (retNodes.count() == 0) {
+	// nothing visible (or a major bug somewhere)
+	return false;
+ }
+
+
+ // if everything worked correct "retNodes" contains only leaf nodes now.
+ // "cell" coordinates are not precise enough for us, so we go a few levels
+ // deeper
+
+ // fallback only
+ pos = BoVector3Float(retNodes[0]->left(), -retNodes[0]->top(), map->cellAverageHeight(retNodes[0]->left(), retNodes[0]->top()));
+
+ QValueVector<EmulatePickRect> rects;
+ for (unsigned int i = 0; i < retNodes.count(); i++) {
+	const int x1 = retNodes[i]->left();
+	const int x2 = retNodes[i]->right() + 1;
+	const int y1 = retNodes[i]->top();
+	const int y2 = retNodes[i]->bottom() + 1;
+	rects.append(EmulatePickRect(
+			BoVector3Float((float)x1, (float)-y1, map->heightAtCorner(x1, y1)),
+			BoVector3Float((float)x2, (float)-y1, map->heightAtCorner(x2, y1)),
+			BoVector3Float((float)x1, (float)-y2, map->heightAtCorner(x1, y2)),
+			BoVector3Float((float)x2, (float)-y2, map->heightAtCorner(x2, y2))
+			));
+ }
+
+ // precision = 1: 1-cell precision
+ // precision = 2: 1/2-cell precision
+ // precision = 3: 1/4-cell precision
+ // precision = 4: 1/8-cell precision
+ // ...
+ const int precision = 4;
+ for (int precisionRun = 0; precisionRun < precision; precisionRun++) {
+	QValueVector<EmulatePickRect> nextRects;
+	nextRects.reserve(rects.count() * 4);
+	const EmulatePickRect* firstVisible = 0;
+	for (unsigned int i = 0; i < rects.count(); i++) {
+		const EmulatePickRect& r = rects[i];
+		BoVector3Float center = r.center();
+#define USE_BOX_NOT_SPHERE 1
+#if USE_BOX_NOT_SPHERE
+		if (!viewFrustum.boxInFrustum(r.boundingBox())) {
+			continue;
+		}
+#else
+		if (!viewFrustum.sphereInFrustum(center, r.radius())) {
+			continue;
+		}
+#endif
+#undef USE_BOX_NOT_SPHERE
+		if (!firstVisible) {
+			firstVisible = &r;
+		}
+
+		BoVector3Float topCenter(center.x(), r.topLeft.y(), (r.topLeft.z() + r.topRight.z()) / 2.0f);
+		BoVector3Float leftCenter(r.topLeft.x(), center.y(), (r.topLeft.z() + r.bottomLeft.z()) / 2.0f);
+		BoVector3Float bottomCenter(center.x(), r.bottomLeft.y(), (r.bottomLeft.z() + r.bottomRight.z()) / 2.0f);
+		BoVector3Float rightCenter(r.topRight.x(), center.y(), (r.topRight.z() + r.bottomRight.z()) / 2.0f);
+
+		// top-left
+		nextRects.append(EmulatePickRect(
+				r.topLeft,
+				topCenter,
+				leftCenter,
+				center
+				));
+
+		// top-right
+		nextRects.append(EmulatePickRect(
+				topCenter,
+				r.topRight,
+				center,
+				rightCenter
+				));
+
+		// bottom-left
+		nextRects.append(EmulatePickRect(
+				leftCenter,
+				center,
+				r.bottomLeft,
+				bottomCenter
+				));
+
+		// bottom-right
+		nextRects.append(EmulatePickRect(
+				center,
+				rightCenter,
+				bottomCenter,
+				r.bottomRight
+				));
+	}
+
+	if (firstVisible) {
+		pos = firstVisible->center();
+	}
+
+	rects = nextRects;
+ }
+
+ if (rects.count() == 0) {
+	*worldPos = pos;
+	return true;
+ }
+
+
+ float x = 0.0f;
+ float y = 0.0f;
+ float z = 0.0f;
+ for (unsigned int i = 0; i < rects.count(); i++) {
+	BoVector3Float center = rects[i].center();
+	x += center.x();
+	y += center.y();
+	z += center.z();
+ }
+ x /= (float)rects.count();
+ y /= (float)rects.count();
+ z /= (float)rects.count();
+
+ *worldPos = BoVector3Float(x, y, z);
+ return true;
+}
+
 void BosonUfoCanvasWidget::paintWidget()
 {
  PROFILE_METHOD;
@@ -688,7 +1082,6 @@ void BosonUfoCanvasWidget::paintWidget()
 	boError() << k_funcinfo << "GL error at the beginning of this method" << endl;
  }
  d->mCanvasRenderer->setCamera(d->mCamera);
- d->mCanvasRenderer->setLocalPlayerIO(d->mLocalPlayerIO);
 
  // Store the original libufo matrices and set our 3d matrices
  glMatrixMode(GL_PROJECTION);
@@ -704,7 +1097,7 @@ void BosonUfoCanvasWidget::paintWidget()
 		d->mGameGLMatrices->viewport()[2],
 		d->mGameGLMatrices->viewport()[3]);
 
- d->mCanvasRenderer->paintGL(d->mCanvas, boViewData->allItemContainers(), d->mEffects);
+ d->mCanvasRenderer->paintGL(boViewData->allItemContainers(), d->mEffects);
 
  glPopAttrib();
 
@@ -719,7 +1112,7 @@ void BosonUfoCanvasWidget::paintWidget()
 }
 
 
-#include "../items/bosonshot.h" // for an explosion hack below
+#include "../gameengine/bosonshot.h" // for an explosion hack below
 BosonItemRenderer* BosonUfoCanvasWidget::createItemRendererFor(const BosonItemContainer* c)
 {
  BO_CHECK_NULL_RET0(c);
