@@ -6,7 +6,7 @@
 
     file              : src/widgets/uwidget.cpp
     begin             : Sun May 13 2001
-    $Id: uwidget.cpp,v 1.13 2005/08/16 18:29:52 abmann Exp $
+    $Id: uwidget.cpp 7041 2005-11-30 23:13:04Z abmann $
  ***************************************************************************/
 
 /***************************************************************************
@@ -59,6 +59,7 @@
 #include "ufo/events/uevents.hpp"
 
 #include "ufo/layouts/ulayoutmanager.hpp"
+#include "ufo/layouts/uboxlayout.hpp"
 
 //#include "ufo/borders/uborder.hpp"
 
@@ -70,12 +71,34 @@
 
 using namespace ufo;
 
+class PaintNotifier
+{
+public:
+	PaintNotifier(UBoUfoWidgetDeleter* d)
+		: m_object(d)
+	{
+		if (m_object) {
+			m_object->startPaint();
+		}
+	}
+	~PaintNotifier()
+	{
+		if (m_object) {
+			m_object->endPaint();
+		}
+	}
+private:
+	UBoUfoWidgetDeleter* m_object;
+};
+
 UFO_IMPLEMENT_DEFAULT_DYNAMIC_CLASS(UWidget, UObject)
 
 UWidget * UWidget::sm_inputFocusWidget = NULL;
 UWidget * UWidget::sm_mouseFocusWidget = NULL;
 UWidget * UWidget::sm_dragWidget = NULL;
 
+static UBoxLayout ufo_defaultLayout;
+
 UWidget::UWidget()
 	: m_context(NULL)
 	, m_model(new UWidgetModel())
@@ -85,16 +108,16 @@ UWidget::UWidget()
 	, m_isFocusable(true)
 	, m_isInValidHierarchy(false)
 	, m_hasInvalidLayout(false)
+	, m_styleHintsDetached(false)
 	, m_eventState(0xffffffff)//MouseEvents | MouseMotionEvents) // FIXME
 	, m_parent(NULL)
 	, m_children()
-	, m_layout(NULL)
+	, m_layout(&ufo_defaultLayout)
 	, m_popupMenu(NULL)
 	, m_cssType("widget")
 	, m_cssClass()
 	, m_style(NULL)
 	, m_styleHints(NULL)
-	, m_styleHintsDetached(false)
 	, m_bounds()
 	, m_clipBounds(URectangle::invalid)
 	, m_cachedRootLocation(UPoint::invalid)
@@ -123,6 +146,7 @@ UWidget::UWidget(ULayoutManager * layout)
 	, m_isFocusable(true)
 	, m_isInValidHierarchy(false)
 	, m_hasInvalidLayout(false)
+	, m_styleHintsDetached(false)
 	, m_eventState(0xffffffff)//MouseEvents | MouseMotionEvents) // FIXME
 	, m_parent(NULL)
 	, m_children()
@@ -132,7 +156,6 @@ UWidget::UWidget(ULayoutManager * layout)
 	, m_cssClass()
 	, m_style(NULL)
 	, m_styleHints(NULL)
-	, m_styleHintsDetached(false)
 	, m_bounds()
 	, m_clipBounds(URectangle::invalid)
 	, m_cachedRootLocation(UPoint::invalid)
@@ -179,13 +202,14 @@ UWidget::~UWidget() {
 }
 
 void
-UWidget::setBoUfoWidgetDeleter(UCollectable* deleter) {
+UWidget::setBoUfoWidgetDeleter(UBoUfoWidgetDeleter* deleter) {
 	delete m_boUfoWidgetDeleter;
 	m_boUfoWidgetDeleter = deleter;
 }
 
 void
 UWidget::setVisible(bool v) {
+	// This explicitely shows/hides the widgets
 	if (!v) {
 		setState(WidgetForceInvisible);
 	} else {
@@ -194,21 +218,7 @@ UWidget::setVisible(bool v) {
 	if (v == testState(WidgetVisible)) {
 		return;
 	}
-/*
-	if (v) {
-		m_isVisible = true;
-		UWidgetEvent * e = new UWidgetEvent(this, UEvent::WidgetShown);
-		processWidgetEvent(e);
-	} else {
-		// widget about to hide
-		UWidgetEvent * e = new UWidgetEvent(this, UEvent::WidgetHidden);
-		processWidgetEvent(e);
-		resetFocus();
-		m_isVisible = false;
-	}
-	invalidate();
-	repaint();
-*/
+
 	if (v && getParent() && !getParent()->isVisible()) {
 		return;
 	}
@@ -220,6 +230,7 @@ UWidget::setChildrenVisible(bool b) {
 	if (b && testState(WidgetForceInvisible)) {
 		return;
 	}
+	// this implicitely shows/hides the window
 	if (b != testState(WidgetVisible)) {
 		if (b) {
 			// show self
@@ -227,7 +238,7 @@ UWidget::setChildrenVisible(bool b) {
 			UWidgetEvent * e = new UWidgetEvent(this, UEvent::WidgetShown);
 			processWidgetEvent(e);
 		} else if (!b) {
-			// show self
+			// hide self
 			setState(WidgetVisible, false);
 			UWidgetEvent * e = new UWidgetEvent(this, UEvent::WidgetHidden);
 			processWidgetEvent(e);
@@ -546,9 +557,13 @@ UWidget::getBackground() const {
 //	return m_context->getUIManager()->getLookAndFeel();//m_lookAndFeel;
 //}
 
+#define DO_UFO_PROFILING 1
 void
 UWidget::paint(UGraphics * g) {
 	if (isVisible()) {
+#ifdef DO_UFO_PROFILING
+		PaintNotifier notifier(m_boUfoWidgetDeleter);
+#endif
 		if (!isValid()) {
 			validate();
 		}
@@ -756,16 +771,18 @@ UWidget::getPopupMenu() const {
 
 
 bool
-UWidget::removeImpl(std::vector<UWidget*>::iterator iter) {
-	if (iter != m_children.end()) {
-		(*iter)->setChildrenVisible(false);
-		(*iter)->removedFromHierarchy();
-		(*iter)->m_parent = NULL;
+UWidget::removeImpl(int index) {
+	if (m_children.size() > index ) {
+		UWidget * ret = m_children[index];
+
+		ret->setChildrenVisible(false);
+		ret->removedFromHierarchy();
+		ret->m_parent = NULL;
 
 		// remove from mem manager
-		releasePointer((*iter));
+		releasePointer(ret);
 
-		m_children.erase(iter);
+		m_children.erase(m_children.begin() + index);
 
 		invalidateTree();
 		return true;
@@ -778,15 +795,12 @@ UWidget::remove(UWidget * w) {
 	std::vector<UWidget*>::iterator iter = std::find(m_children.begin(),
 		m_children.end(), w);
 
-	return removeImpl(iter);
+	return removeImpl(iter - m_children.begin());
 }
 
 bool
 UWidget::remove(unsigned int n) {
-	if (m_children.size() > n ) {
-		return removeImpl(m_children.begin() + n);
-	}
-	return false;
+	return removeImpl(n);
 }
 
 UWidget *
@@ -796,7 +810,7 @@ UWidget::removeAndReturn(unsigned int n) {
 		if (ret) {
 			ret->reference();
 		}
-		removeImpl(m_children.begin() + n);
+		removeImpl(n);
 		return ret;
 	}
 	return NULL;
@@ -804,12 +818,14 @@ UWidget::removeAndReturn(unsigned int n) {
 
 unsigned int
 UWidget::removeAll() {
-	unsigned int size = m_children.size();
-
-	while (removeImpl(m_children.begin())) {}
+	unsigned int count = 0;
+	while (removeImpl(0)) {
+		count++;
+	}
+	// count should be exactly the former widget count ...
 
 	invalidate();
-	return size;
+	return count;
 }
 
 
@@ -1179,7 +1195,7 @@ UWidget::dispatchEvent(UEvent * e) {
 		processEvent(e);
 		propagate = !e->isConsumed();
 	}
-#if 0
+#if 0 // libufo event code (not ULayeredWidget compatible)
 	if (propagate && m_parent && dynamic_cast<UInputEvent*>(e) &&
 			e->getType() != UEvent::MouseEntered &&
 			e->getType() != UEvent::MouseExited) {
@@ -1422,8 +1438,8 @@ UWidget::setIndexOf(UWidget * w, int index) {
 		} else {
 			m_children.insert(m_children.begin() + index, w);
 		}
-		int min_pos = std::min(index, old_pos);
-		for (int i = min_pos; i < m_children.size(); ++i) {
+		unsigned int min_pos = std::min(index, old_pos);
+		for (unsigned int i = min_pos; i < m_children.size(); ++i) {
 			getWidget(i)->processWidgetEvent(
 				new UWidgetEvent(getWidget(i), UEvent::WidgetZOrderChanged)
 			);
@@ -1791,7 +1807,7 @@ UWidget::notifyKeyBindingAction(const UKeyStroke & ks, UKeyEvent * e, InputCondi
 
 
 static std::list<std::pair<UKeyStroke, UWidget*> > ufo_s_shortcuts;
-static int ufo_s_oldIndex = 0;
+static unsigned int ufo_s_oldIndex = 0;
 static UKeyStroke ufo_s_oldStroke;
 
 void
